@@ -18,8 +18,8 @@ module ActiveSupport #:nodoc:
           replace(reverse_merge(other_hash))
         end
 
-        alias_method :reverse_update, :reverse_merge
+        alias_method :reverse_update, :reverse_merge!
       end
     end
   end
-end
\ No newline at end of file
+end
