@@ -8,10 +8,12 @@ class Time
   end
 end
 
-require File.dirname(__FILE__) + '/time/calculations'
-require File.dirname(__FILE__) + '/time/conversions'
+require 'active_support/core_ext/time/behavior'
+require 'active_support/core_ext/time/calculations'
+require 'active_support/core_ext/time/conversions'
 
 class Time#:nodoc:
+  include ActiveSupport::CoreExtensions::Time::Behavior
   include ActiveSupport::CoreExtensions::Time::Calculations
   include ActiveSupport::CoreExtensions::Time::Conversions
 end
